@@ -2,6 +2,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <time.h>
+#include <corosync/confdb.h>
 #include <netinet/in.h>
 #include "copyright.cf"
 #include "libcman.h"
@@ -9,7 +10,7 @@
 
 #define DEFAULT_CONFIG_MODULE "xmlconfig"
 
-#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:C:VAPwfqah?Xd::")
+#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:C:VAPwfqah?XD::Sd::")
 #define OP_JOIN			1
 #define OP_LEAVE		2
 #define OP_EXPECTED		3
@@ -21,8 +22,6 @@
 #define OP_NODES		9
 #define OP_SERVICES		10
 #define OP_DEBUG		11
-#define OP_DUMP_OBJDB		12
-
 
 static void print_usage(int subcmd)
 {
@@ -54,10 +53,13 @@ static void print_usage(int subcmd)
 		printf("  -w               Wait until node has joined a cluster\n");
 		printf("  -q               Wait until the cluster is quorate\n");
 		printf("  -t               Maximum time (in seconds) to wait\n");
-		printf("  -k <file>        Private key file for AIS communications\n");
-		printf("  -P               Don't set aisexec to realtime priority\n");
+		printf("  -k <file>        Private key file for Corosync communications\n");
+		printf("  -P               Don't set corosync to realtime priority\n");
 		printf("  -X               Use internal cman defaults for configuration\n");
 		printf("  -A               Don't load openais services\n");
+		printf("  -D <fail,warn,none> What to do about the config. Default (without -D) is to validate\n");
+		printf("                   the config. with -D no validation will be done. -Dwarn will print errors\n");
+		printf("                   but allow the operation to continue\n");
 		printf("\n");
 	}
 
@@ -103,7 +105,6 @@ static void print_usage(int subcmd)
 
 	if (!subcmd || subcmd == OP_NODES) {
 		printf("nodes              Show local record of cluster nodes\n");
-		printf("  -f                 Also show when node was last fenced\n");
 		printf("  -a                 Also show node address(es)\n");
 		printf("  -n <nodename>      Only show information for specific node\n");
 		printf("  -F <format>        Specify output format (see man page)\n");
@@ -118,6 +119,10 @@ static void print_usage(int subcmd)
 	if (!subcmd || subcmd == OP_VERSION) {
 		printf("version\n");
 		printf("  -r <config>      A new config version to set on all members\n");
+		printf("  -D <fail,warn,none> What to do about the config. Default (without -D) is to validate\n");
+		printf("                   the config. with -D no validation will be done. -Dwarn will print errors\n");
+		printf("                   but allow the operation to continue\n");
+		printf("  -S               Don't run ccs_sync to distribute cluster.conf (if appropriate)\n");
 		printf("\n");
 	}
 }
@@ -239,8 +244,10 @@ static void show_status(void)
 		printf(" Error");
 	if (einfo->ei_flags & CMAN_EXTRA_FLAG_DISALLOWED)
 		printf(" DisallowedNodes");
+	if (einfo->ei_flags & CMAN_EXTRA_FLAG_DISALLOWED_ENABLED)
+		printf(" DisallowedEnabled");
 	if (einfo->ei_flags & CMAN_EXTRA_FLAG_DIRTY)
-		printf(" Dirty");
+		printf(" HaveState");
 	printf(" \n");
 
 	printf("Ports Bound: ");
@@ -257,7 +264,7 @@ static void show_status(void)
 	node.cn_name[0] = 0;
 	if (cman_get_node(h, CMAN_NODEID_US, &node) == 0) {
 		printf("Node name: %s\n", node.cn_name);
-		printf("Node ID: %d\n", node.cn_nodeid);
+		printf("Node ID: %u\n", node.cn_nodeid);
 	}
 
 	printf("Multicast addresses: ");
@@ -329,6 +336,12 @@ static int get_format_opt(const char *opt)
 		return FMT_TYPE;
 	if (!strcmp(opt, "addr"))
 		return FMT_ADDR;
+	if (!strcmp(opt, "votes"))
+		return FMT_VOTES;
+	if (!strcmp(opt, "exp"))
+		return FMT_EXP;
+	if (!strcmp(opt, "state"))
+		return FMT_STATE;
 
 	return FMT_NONE;
 }
@@ -337,10 +350,13 @@ static int get_format_opt(const char *opt)
 static void print_node(commandline_t *comline, cman_handle_t h, int *format, struct cman_node *node)
 {
 	char member_type;
-	struct tm *ftime;
 	struct tm *jtime;
+	int numaddrs;
+	struct cman_node_address addrs[MAX_INTERFACES];
 	char jstring[1024];
 	int i,j,k;
+	unsigned int tmpid;
+	cman_node_extra_t enode;
 
 	if (comline->num_nodenames > 0) {
 		if (node_filter(comline, node->cn_name) == 0) {
@@ -363,6 +379,15 @@ static void print_node(commandline_t *comline, cman_handle_t h, int *format, str
 		break;
 	}
 
+	/* Make the name more friendly if cman can't find it in cluster.conf
+	 *  (we really don't want corosync to look up names in DNS so it invents them)
+	 */
+	if (sscanf(node->cn_name, "Node%u", &tmpid) == 1 && tmpid == node->cn_nodeid) {
+		if (!cman_get_node_addrs(h, node->cn_nodeid, MAX_INTERFACES, &numaddrs, addrs) && numaddrs) {
+			getnameinfo((struct sockaddr *)addrs[0].cna_address, addrs[0].cna_addrlen, node->cn_name, sizeof(node->cn_name), NULL, 0, NI_NAMEREQD);
+		}
+	}
+
 	jtime = localtime(&node->cn_jointime.tv_sec);
 	if (node->cn_jointime.tv_sec && node->cn_member)
 		strftime(jstring, sizeof(jstring), "%F %H:%M:%S", jtime);
@@ -370,32 +395,11 @@ static void print_node(commandline_t *comline, cman_handle_t h, int *format, str
 		strcpy(jstring, "                   ");
 
 	if (!comline->format_opts) {
-		printf("%4d   %c  %5d   %s  %s\n",
+		printf("%4u   %c  %5d   %s  %s\n",
 		       node->cn_nodeid, member_type,
 		       node->cn_incarnation, jstring, node->cn_name);
 	}
 
-	if (comline->fence_opt && !comline->format_opts) {
-		char agent[255];
-		uint64_t fence_time;
-		int fenced;
-
-		if (!cman_get_fenceinfo(h, node->cn_nodeid, &fence_time, &fenced, agent)) {
-			if (fence_time) {
-				time_t fence_time_t = (time_t)fence_time;
-				ftime = localtime(&fence_time_t);
-				strftime(jstring, sizeof(jstring), "%F %H:%M:%S", ftime);
-				printf("       Last fenced:   %-15s by %s\n", jstring, agent);
-			}
-			if (!node->cn_member && node->cn_incarnation && !fenced) {
-				printf("       Node has not been fenced since it went down\n");
-			}
-		}
-	}
-
-	int numaddrs;
-	struct cman_node_address addrs[MAX_INTERFACES];
-
 	if (comline->addresses_opt || comline->format_opts) {
 		if (!cman_get_node_addrs(h, node->cn_nodeid, MAX_INTERFACES, &numaddrs, addrs) &&
 		    numaddrs)
@@ -412,13 +416,13 @@ static void print_node(commandline_t *comline, cman_handle_t h, int *format, str
 		}
 	}
 
-	if (comline->format_opts) {
+	if (comline->format_opts && (cman_get_node_extra(h, node->cn_nodeid, &enode) == 0)) {
 		for (j = 0; j < MAX_FORMAT_OPTS; j++) {
 			switch (format[j]) {
 			case FMT_NONE:
 				break;
 			case FMT_ID:
-				printf("%d ", node->cn_nodeid);
+				printf("%u ", node->cn_nodeid);
 				break;
 			case FMT_NAME:
 				printf("%s ", node->cn_name);
@@ -426,6 +430,35 @@ static void print_node(commandline_t *comline, cman_handle_t h, int *format, str
 			case FMT_TYPE:
 				printf("%c ", member_type);
 				break;
+			case FMT_VOTES:
+				printf("%d ", enode.cnx_votes);
+				break;
+			case FMT_EXP:
+				printf("%d ", enode.cnx_expected_votes);
+				break;
+			case FMT_STATE:
+				switch (enode.cnx_state)
+				{
+				case CLUSTER_NODESTATE_JOINING:
+					printf("Joining ");
+					break;
+				case CLUSTER_NODESTATE_MEMBER:
+					printf("Member ");
+					break;
+				case CLUSTER_NODESTATE_DEAD:
+					printf("Dead ");
+					break;
+				case CLUSTER_NODESTATE_LEAVING:
+					printf("Leaving ");
+					break;
+				case CLUSTER_NODESTATE_DISALLOWED:
+					printf("Disallowed ");
+					break;
+				default:
+					printf("Unknown ");
+					break;
+				}
+				break;
 			case FMT_ADDR:
 				for (k = 0; k < numaddrs; k++) {
 					print_address(addrs[k].cna_address);
@@ -521,9 +554,9 @@ static int show_services(void)
 }
 
 
-char *cman_error(int err)
+const char *cman_error(int err)
 {
-	char *die_error;
+	const char *die_error;
 
 	switch (errno) {
 	case ENOTCONN:
@@ -561,11 +594,18 @@ static void leave(commandline_t *comline)
 	}
 
 	result = cman_shutdown(h, flags);
-	if (result) {
+	if (result && !comline->wait_opt) {
 		die("Error leaving cluster: %s", cman_error(errno));
 	}
-
 	cman_finish(h);
+
+	/* Wait until cman shuts down */
+	if (comline->wait_opt) {
+		while ( (h = cman_admin_init(NULL)) ) {
+			cman_finish(h);
+			sleep(1);
+		}
+	}
 }
 
 static void set_expected(commandline_t *comline)
@@ -609,27 +649,141 @@ static void set_votes(commandline_t *comline)
 	cman_finish(h);
 }
 
+static int validate_config(commandline_t *comline)
+{
+	struct stat st;
+	char command[PATH_MAX];
+	char validator[PATH_MAX];
+	int cmd_res;
+
+	/* Look for ccs_config_validate */
+	snprintf(validator, sizeof(validator), "%s/ccs_config_validate", SBINDIR);
+	if (stat(validator, &st) != 0 || !(st.st_mode & S_IXUSR)) {
+		fprintf(stderr, "Cannot find ccs_config_validate, configuration was not checked but assumed to be OK.\n");
+		return 0;
+	}
+
+	snprintf(command, sizeof(command), "%s -q", validator);
+
+	if (comline->verbose > 1)
+		printf("calling '%s'\n", command);
+
+	cmd_res = system(command);
+
+	return cmd_res;
+}
+
+/* Here we set the COROSYNC_ variables that might be needed by the corosync
+   configuration modules. We just put them into the environment and leave
+   them for the sub-process to pick up.
+   'config_modules' is returned separately because it's needed internally to
+   and it saves the caller from extracting it all over again.
+   We only return 0 (success) if config_modules is returned as without that
+   the caller can't really do anything at all.
+*/
+static int get_config_variables(commandline_t *comline, char **config_modules)
+{
+	int res;
+	int got_iface = 1;
+	char key_name[1024];
+	size_t key_name_len;
+	char key_value[1024];
+	size_t key_value_len;
+	hdb_handle_t confdb_handle;
+	hdb_handle_t cmanp_handle;
+	confdb_callbacks_t callbacks = {
+		.confdb_key_change_notify_fn = NULL,
+		.confdb_object_create_change_notify_fn = NULL,
+		.confdb_object_delete_change_notify_fn = NULL
+	};
+
+	*config_modules = NULL;
+	res = confdb_initialize (&confdb_handle, &callbacks);
+	if (res != CS_OK)
+		return 0;
+
+	res = confdb_object_find_start(confdb_handle, OBJECT_PARENT_HANDLE);
+	if (res != CS_OK)
+		goto finish;
+
+	res = confdb_object_find(confdb_handle, OBJECT_PARENT_HANDLE, "cman_private", strlen("cman_private"), &cmanp_handle);
+	if (res != CS_OK)
+		goto finish;
+
+	res = confdb_key_iter_start(confdb_handle, cmanp_handle);
+	if (res != CS_OK)
+		goto finish;
+
+	while ( (res = confdb_key_iter(confdb_handle, cmanp_handle, key_name, &key_name_len,
+				       key_value, &key_value_len)) == CS_OK) {
+		key_name[key_name_len] = '\0';
+		key_value[key_value_len] = '\0';
+
+		setenv(key_name, key_value, 1);
+		if (strcmp(key_name, "COROSYNC_DEFAULT_CONFIG_IFACE") == 0) {
+			*config_modules = strdup(key_value);
+			got_iface = 0;
+		}
+	}
+
+finish:
+	confdb_finalize(confdb_handle);
+	return got_iface;
+}
+
 static void version(commandline_t *comline)
 {
 	struct cman_version ver;
 	cman_handle_t h;
 	int result;
+	char *config_modules = NULL;
 
 	h = open_cman_handle(1);
 
 	if ((result = cman_get_version(h, &ver)))
 		die("can't get version: %s", cman_error(errno));
 
-	if (!comline->config_version) {
+	if (!comline->config_version_opt) {
 		printf("%d.%d.%d config %d\n", ver.cv_major, ver.cv_minor, ver.cv_patch,
 		       ver.cv_config);
 		goto out;
 	}
 
-	ver.cv_config = comline->config_version;
+	if (comline->verbose)
+	        printf("Getting config variables\n");
+	if (get_config_variables(comline, &config_modules))
+	        die("cannot get COROSYNC_DEFAULT_CONFIG_IFACE");
 
-	if ((result = cman_set_version(h, &ver)))
-		die("can't set version: %s", cman_error(errno));
+	/* By default we validate the configuration first */
+	if (comline->config_validate_opt != VALIDATE_NONE) {
+
+		if (comline->verbose)
+			printf("Validating configuration\n");
+		if (validate_config(comline) &&
+		    comline->config_validate_opt == VALIDATE_FAIL)
+			die("Not reloading, configuration is not valid\n");
+	}
+
+	/* We don't bother looking for ccs_sync here - just assume it's in /usr/bin and
+	   that it exists. If this is not true then then user can choose to bypass
+	   the disibution and do it themselves.
+	   Note that ccs_sync might prompt on stderr for passwords the first time it is
+	   run.
+	*/
+	if (strstr(config_modules, "xmlconfig") && !comline->nosync_opt) {
+		if (comline->verbose > 1)
+			printf("calling ccs_sync\n");
+		result = system("/usr/bin/ccs_sync");
+	}
+	else {
+		if (comline->verbose)
+			printf("Telling cman the new version number\n");
+
+		ver.cv_config = comline->config_version;
+
+		if ((result = cman_set_version(h, &ver)))
+			die("can't set version: %s", cman_error(errno));
+	}
  out:
 	cman_finish(h);
 }
@@ -699,20 +853,6 @@ static void set_debuglog(commandline_t *comline)
 	cman_finish(h);
 }
 
-#ifdef DEBUG
-static void dump_objdb(commandline_t *comline)
-{
-	cman_handle_t h;
-
-	h = open_cman_handle(1);
-
-	if (cman_dump_objdb(h, comline->filename))
-		perror("dump objdb failed");
-
-	cman_finish(h);
-}
-#endif
-
 static int get_int_arg(char argopt, char *arg)
 {
 	char *tmp;
@@ -733,10 +873,9 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 {
 	int cont = TRUE;
 	int optchar, i;
+	int suboptchar;
 	int show_help = 0;
 
-	comline->config_lcrso=DEFAULT_CONFIG_MODULE;
-
 	while (cont) {
 		optchar = getopt(argc, argv, OPTION_STRING);
 
@@ -746,14 +885,38 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			comline->multicast_addr = strdup(optarg);
 			break;
 
-		case 'f':
-			comline->fence_opt = 1;
-			break;
-
 		case 'a':
 			comline->addresses_opt = 1;
 			break;
 
+		case 'D':
+			/* Just look at the upper-cased version of the first letter of the argument */
+			if (optarg) {
+				suboptchar = optarg[0] & 0x5F;
+				switch (suboptchar)
+				{
+				case 'F':
+					comline->config_validate_opt = VALIDATE_FAIL;
+					break;
+				case 'W':
+					comline->config_validate_opt = VALIDATE_WARN;
+					break;
+				case 'N':
+					comline->config_validate_opt = VALIDATE_NONE;
+					break;
+				default:
+					die("invalid option to -D, it should be 'force', 'warn' or 'none'\n");
+					break;
+				}
+			}
+			else {
+				comline->config_validate_opt = VALIDATE_NONE;
+			}
+			break;
+		case 'S':
+			comline->nosync_opt = 1;
+			break;
+
 		case 'n':
 		        i = comline->num_nodenames;
 			if (i >= MAX_INTERFACES)
@@ -876,6 +1039,9 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 		};
 	}
 
+	if (comline->config_lcrso == NULL)
+		comline->config_lcrso = strdup(DEFAULT_CONFIG_MODULE);
+
 	while (optind < argc) {
 		if (strcmp(argv[optind], "join") == 0) {
 			if (comline->operation)
@@ -921,18 +1087,6 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			if (comline->operation)
 				die("can't specify two operations");
 			comline->operation = OP_DEBUG;
-#ifdef DEBUG
-		} else if (strcmp(argv[optind], "dump-db") == 0) {
-			if (comline->operation)
-				die("can't specify two operations");
-			comline->operation = OP_DUMP_OBJDB;
-			if (!argv[optind+1])
-				die("no filename given");
-			comline->filename = strdup(argv[optind+1]);
-			if (comline->filename[0] != '/')
-				die("dump filename must be an absolute path");
-			optind++;
-#endif
 		} else if (strcmp(argv[optind], "remove") == 0) {
 			comline->remove = TRUE;
 		} else if (strcmp(argv[optind], "force") == 0) {
@@ -977,10 +1131,43 @@ static void check_arguments(commandline_t *comline)
 		die("timeout is only appropriate with wait");
 }
 
+
+static void do_join(commandline_t *comline, char *envp[])
+{
+	int ret;
+
+	check_arguments(comline);
+
+	if (comline->timeout) {
+		signal(SIGALRM, sigalarm_handler);
+		alarm(comline->timeout);
+	}
+
+	/* By default we validate the configuration first */
+	if (comline->config_validate_opt != VALIDATE_NONE) {
+
+		if (comline->verbose)
+			printf("Validating configuration\n");
+
+		if (validate_config(comline) &&
+		    comline->config_validate_opt == VALIDATE_FAIL)
+			die("Not joining, configuration is not valid\n");
+	}
+
+	join(comline, envp);
+	if (comline->wait_opt || comline->wait_quorate_opt) {
+		do {
+			ret = cluster_wait(comline);
+			if (ret == ENOTCONN)
+				join(comline, envp);
+
+		} while (ret == ENOTCONN);
+	}
+}
+
 int main(int argc, char *argv[], char *envp[])
 {
 	commandline_t comline;
-	int ret;
 
 	prog_name = argv[0];
 
@@ -990,22 +1177,7 @@ int main(int argc, char *argv[], char *envp[])
 
 	switch (comline.operation) {
 	case OP_JOIN:
-		check_arguments(&comline);
-
-		if (comline.timeout) {
-			signal(SIGALRM, sigalarm_handler);
-			alarm(comline.timeout);
-		}
-
-		join(&comline, envp);
-		if (comline.wait_opt || comline.wait_quorate_opt) {
-			do {
-				ret = cluster_wait(&comline);
-				if (ret == ENOTCONN)
-					join(&comline, envp);
-
-			} while (ret == ENOTCONN);
-		}
+		do_join(&comline, envp);
 		break;
 
 	case OP_LEAVE:
@@ -1054,11 +1226,6 @@ int main(int argc, char *argv[], char *envp[])
 	case OP_DEBUG:
 		set_debuglog(&comline);
 		break;
-#ifdef DEBUG
-	case OP_DUMP_OBJDB:
-		dump_objdb(&comline);
-		break;
-#endif
 	}
 
 	exit(EXIT_SUCCESS);
