@@ -60,6 +60,7 @@ extern int two_node;
        unsigned int quorumdev_poll=DEFAULT_QUORUMDEV_POLL;
        unsigned int shutdown_timeout=DEFAULT_SHUTDOWN_TIMEOUT;
        unsigned int ccsd_poll_interval=DEFAULT_CCSD_POLL;
+       unsigned int enable_disallowed=DEFAULT_DISALLOWED;
 static int cluster_is_quorate;
        char cluster_name[MAX_CLUSTER_NAME_LEN+1];
 static char nodename[MAX_CLUSTER_MEMBER_NAME_LEN+1];
@@ -75,6 +76,7 @@ static struct corosync_api_v1 *corosync;
 static corosync_timer_handle_t ccsd_timer;
 static unsigned int wanted_config_version;
 static int config_error;
+static int local_first_trans;
 
 static corosync_timer_handle_t shutdown_timer;
 static struct connection *shutdown_con;
@@ -133,6 +135,9 @@ static int have_disallowed(void)
 {
 	struct cluster_node *node;
 
+	if (!enable_disallowed)
+	        return 0;
+
 	list_iterate_items(node, &cluster_members_list) {
 		if (node->state == NODESTATE_AISONLY)
 			return 1;
@@ -483,7 +488,7 @@ static int do_cmd_set_version(char *cmdbuf, int *retlen)
 	/* If the passed-in version number is 0 then read the file now, then
 	 * tell the other nodes to look for that version number.
 	 * That means we also have to send the notification here, because it will
-	 * beskipped when we get our own RECONFIGURE message back as the version
+	 * be skipped when we get our own RECONFIGURE message back, as the version
 	 * number will match.
 	 */
 	if (!version->config) {
@@ -565,6 +570,8 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 		einfo->flags |= CMAN_EXTRA_FLAG_UNCOUNTED;
 	if (us->flags & NODE_FLAGS_DIRTY)
 		einfo->flags |= CMAN_EXTRA_FLAG_DIRTY;
+	if (enable_disallowed)
+		einfo->flags |= CMAN_EXTRA_FLAG_DISALLOWED_ENABLED;
 
 	ptr = einfo->addresses;
 
@@ -992,7 +999,8 @@ static void check_shutdown_status(void)
 	/* All replies safely gathered in ? */
 	if (shutdown_yes + shutdown_no >= shutdown_expected) {
 
-		corosync->timer_delete(shutdown_timer);
+		if (shutdown_timer)
+			corosync->timer_delete(shutdown_timer);
 
 		if (shutdown_yes >= shutdown_expected ||
 		    shutdown_flags & SHUTDOWN_ANYWAY) {
@@ -1070,9 +1078,8 @@ static int do_cmd_try_shutdown(struct connection *con, char *cmdbuf)
 
 	/* If no-one is listening for events then we can just go down now */
 	if (shutdown_expected == 0) {
-		quit_threads = 1;
-		send_leave(CLUSTER_LEAVEFLAG_DOWN);
-		return 0;
+		shutdown_timer = 0;
+		check_shutdown_status();
 	}
 	else {
 
@@ -1085,6 +1092,7 @@ static int do_cmd_try_shutdown(struct connection *con, char *cmdbuf)
 
 		return -EWOULDBLOCK;
 	}
+	return 0;
 }
 
 static int do_cmd_register_quorum_device(char *cmdbuf, int *retlen)
@@ -1098,38 +1106,49 @@ static int do_cmd_register_quorum_device(char *cmdbuf, int *retlen)
 	if (!we_are_a_cluster_member)
 		return -ENOENT;
 
-	if (quorum_device)
-                return -EBUSY;
-
 	if (strlen(name) > MAX_CLUSTER_MEMBER_NAME_LEN)
 		return -EINVAL;
 
+	/* Allow re-registering of a quorum device if the name is the same */
+	if (quorum_device && strcmp(name, quorum_device->name))
+                return -EBUSY;
+
 	if (find_node_by_name(name))
                 return -EALREADY;
 
 	memcpy(&votes, cmdbuf, sizeof(int));
 
-	quorum_device = malloc(sizeof(struct cluster_node));
-        if (!quorum_device)
-                return -ENOMEM;
-        memset(quorum_device, 0, sizeof(struct cluster_node));
+	/* A new quorum device */
+	if (!quorum_device)
+	{
+		quorum_device = malloc(sizeof(struct cluster_node));
+		if (!quorum_device)
+			return -ENOMEM;
+		memset(quorum_device, 0, sizeof(struct cluster_node));
 
-        quorum_device->name = malloc(strlen(name) + 1);
-        if (!quorum_device->name) {
-                free(quorum_device);
-                quorum_device = NULL;
-                return -ENOMEM;
-        }
+		quorum_device->name = malloc(strlen(name) + 1);
+		if (!quorum_device->name) {
+			free(quorum_device);
+			quorum_device = NULL;
+			return -ENOMEM;
+		}
 
-        strcpy(quorum_device->name, name);
+		strcpy(quorum_device->name, name);
+		quorum_device->state = NODESTATE_DEAD;
+		gettimeofday(&quorum_device->join_time, NULL);
+
+		/* Keep this list valid so it doesn't confuse other code */
+		list_init(&quorum_device->addr_list);
+		log_printf(LOG_INFO, "quorum device registered\n");
+	}
+	else
+	{
+		log_printf(LOG_INFO, "quorum device re-registered\n");
+	}
+
+	/* Update votes even if it existed before */
         quorum_device->votes = votes;
-        quorum_device->state = NODESTATE_DEAD;
-	gettimeofday(&quorum_device->join_time, NULL);
 
-        /* Keep this list valid so it doesn't confuse other code */
-        list_init(&quorum_device->addr_list);
-
-	log_printf(LOG_INFO, "quorum device registered\n");
         return 0;
 }
 
@@ -1714,6 +1733,7 @@ void send_transition_msg(int last_memb_count, int first_trans)
 	int len = sizeof(struct cl_transmsg);
 
 	we_are_a_cluster_member = 1;
+	local_first_trans = first_trans;
 
 	log_printf(LOGSYS_LEVEL_DEBUG, "memb: sending TRANSITION message. cluster_name = %s\n", cluster_name);
 	msg->cmd = CLUSTER_MSG_TRANSITION;
@@ -1884,9 +1904,10 @@ static void do_process_transition(int nodeid, char *data)
 		return;
 	}
 
-	/* If the remote node can see AISONLY nodes then we can't join as we don't
-	   know the full state */
-	if (msg->flags & NODE_FLAGS_SEESDISALLOWED && !have_disallowed()) {
+	/* If the remote node can see AISONLY nodes and we want to join,
+	   then we can't, as we don't know the full state */
+	if (enable_disallowed && 
+	    local_first_trans && msg->flags & NODE_FLAGS_SEESDISALLOWED && !have_disallowed()) {
 		/* Must use syslog directly here or the message will never arrive */
 		syslog(LOG_CRIT, "CMAN: Joined a cluster with disallowed nodes. must die");
 		cman_finish();
@@ -1908,50 +1929,56 @@ static void do_process_transition(int nodeid, char *data)
 
 	/* Newer nodes 6.1.0 onwards, set the DIRTY flag if they have state. If the new node has been down
 	   and has state then we mark it disallowed because we cannot merge stateful nodes */
-	if ( (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN)) ||
-	     (msg->flags & NODE_FLAGS_DIRTY && msg->first_trans && !node->us && (us->flags & NODE_FLAGS_DIRTY))) {
-		/* Don't duplicate messages */
-		if (node->state != NODESTATE_AISONLY) {
-			if (cluster_is_quorate) {
-				log_printf(LOG_CRIT, "Killing node %s because it has rejoined the cluster with existing state", node->name);
-				node->state = NODESTATE_AISONLY;
-				send_kill(nodeid, CLUSTER_KILL_REJOIN);
-			}
-			else {
-				log_printf(LOG_CRIT, "Node %s not joined to cman because it has existing state", node->name);
-				node->state = NODESTATE_AISONLY;
+	if (enable_disallowed) {
+		if ( (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN)) ||
+		     (msg->flags & NODE_FLAGS_DIRTY && msg->first_trans && !node->us && (us->flags & NODE_FLAGS_DIRTY))) {
+			/* Don't duplicate messages */
+			if (node->state != NODESTATE_AISONLY) {
+				if (cluster_is_quorate) {
+					log_printf(LOG_CRIT, "Killing node %s because it has rejoined the cluster with existing state", node->name);
+					node->state = NODESTATE_AISONLY;
+					send_kill(nodeid, CLUSTER_KILL_REJOIN);
+				}
+				else {
+					log_printf(LOG_CRIT, "Node %s not joined to cman because it has existing state", node->name);
+					node->state = NODESTATE_AISONLY;
+				}
 			}
+			return;
 		}
-		return;
-	}
 
-        /* This is for older nodes. If the join_time of the node matches that already stored AND
-	   the node has been down, then we kill it as this must be a rejoin */
-	if (msg->minor_version == 0 &&
-	    msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
-		/* Don't duplicate messages */
-		if (node->state != NODESTATE_AISONLY) {
-			if (cluster_is_quorate) {
-				log_printf(LOG_CRIT, "Killing node %s because it has rejoined the cluster without cman_tool join", node->name);
-				node->state = NODESTATE_AISONLY;
-				send_kill(nodeid, CLUSTER_KILL_REJOIN);
-			}
-			else {
-				log_printf(LOG_CRIT, "Node %s not joined to cman because it has rejoined an inquorate cluster", node->name);
-				node->state = NODESTATE_AISONLY;
+		/* This is for older nodes. If the join_time of the node matches that already stored AND
+		   the node has been down, then we kill it as this must be a rejoin */
+		if (msg->minor_version == 0 &&
+		    msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
+			/* Don't duplicate messages */
+			if (node->state != NODESTATE_AISONLY) {
+				if (cluster_is_quorate) {
+					log_printf(LOG_CRIT, "Killing node %s because it has rejoined the cluster without cman_tool join", node->name);
+					node->state = NODESTATE_AISONLY;
+					send_kill(nodeid, CLUSTER_KILL_REJOIN);
+				}
+				else {
+					log_printf(LOG_CRIT, "Node %s not joined to cman because it has rejoined an inquorate cluster", node->name);
+					node->state = NODESTATE_AISONLY;
+				}
 			}
+			return;
+		}
+		else {
+			node->cman_join_time = msg->join_time;
+			add_ais_node(nodeid, incarnation, num_ais_nodes);
 		}
-		return;
 	}
 	else {
-		node->cman_join_time = msg->join_time;
-		add_ais_node(nodeid, incarnation, num_ais_nodes);
+	        add_ais_node(nodeid, incarnation, num_ais_nodes);
 	}
 
-	/* If the cluster already has some AISONLY nodes then we can't make
-	   sense of the membership. So the new node has to also be AISONLY
-	   until we are consistent again */
-	if (have_disallowed() && !node->us)
+	/* If the new node is joining and the existing cluster already has some AISONLY
+	   nodes then we can't make sense of the membership.
+	   So the new node has to also be AISONLY until we are consistent again */
+	if (enable_disallowed &&
+	    msg->first_trans && !node->us && have_disallowed())
 		node->state = NODESTATE_AISONLY;
 
 	node->flags = msg->flags; /* This will clear the BEENDOWN flag of course */
