@@ -907,7 +907,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 	int low_id, bid_pending = 0, score, score_max, score_req,
 	    upgrade = 0, count, errors, error_cycles = 0;
 	memb_mask_t mask, master_mask;
-	struct timeval maxtime, oldtime, newtime, diff, sleeptime, interval;
+	struct timeval maxtime, oldtime, newtime, diff, sleeptime, interval, rd_lastok, wr_lastok;
 
 	ctx->qc_status = S_NONE;
 	
@@ -917,6 +917,12 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 	interval.tv_usec = 0;
 	interval.tv_sec = ctx->qc_interval;
 	
+	rd_lastok.tv_usec = 0;
+	rd_lastok.tv_sec = 0;
+	
+	wr_lastok.tv_usec = 0;
+	wr_lastok.tv_sec = 0;
+	
 	get_my_score(&score, &score_max);
 	if (score_max < ctx->qc_scoremin) {
 		logt_print(LOG_WARNING, "Minimum score (%d) is impossible to "
@@ -933,9 +939,10 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 
 		/* XXX this was getuptime() in clumanager */
 		get_time(&oldtime, (ctx->qc_flags&RF_UPTIME));
-		
+
 		/* Read everyone else's status */
-		errors = read_node_blocks(ctx, ni, max);
+		if ((errors = read_node_blocks(ctx, ni, max)) == 0 ) 
+			get_time(&rd_lastok, ctx->qc_flags&RF_UPTIME);
 
 		/* Check for node transitions */
 		check_transitions(ctx, ni, max, mask);
@@ -1110,6 +1117,8 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 			logt_print(LOG_ERR, "Error writing to quorum disk\n");
 			errors++; /* this value isn't really used 
 				     at this point */
+ 		} else {
+ 			get_time(&wr_lastok, ctx->qc_flags&RF_UPTIME);
 		}
 
 		/* write out our local status */
@@ -1118,11 +1127,44 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 		/* Cycle. We could time the loop and sleep
 		   (interval-looptime), but this is fine for now.*/
 		get_time(&newtime, ctx->qc_flags&RF_UPTIME);
-		_diff_tv(&diff, &oldtime, &newtime);
 		
+ 		/*
+		 * Reboot if the last successful hearbeat was longer ago than interval*TKO_COUNT
+		 */
+		_diff_tv(&diff, &wr_lastok, &newtime);
+		if (_cmp_tv(&maxtime, &diff) == 1 &&
+		    ctx->qc_flags & RF_IOTIMEOUT) {
+			logt_print(LOG_EMERG, "Failed to send a heartbeat "
+				   "within %d second%s (%d.%06d) - REBOOTING\n",
+				   (int)maxtime.tv_sec,
+				   maxtime.tv_sec==1?"":"s",
+				   (int)diff.tv_sec,
+				   (int)diff.tv_usec);
+			if (!(ctx->qc_flags & RF_DEBUG)) 
+				reboot(RB_AUTOBOOT);
+		}
+
+ 		/*
+		 * Reboot if the last successful hearbeat was longer ago than interval*TKO_COUNT
+		 */
+		_diff_tv(&diff, &rd_lastok, &newtime);
+		if (_cmp_tv(&maxtime, &diff) == 1 &&
+		    ctx->qc_flags & RF_IOTIMEOUT) {
+			logt_print(LOG_EMERG,
+				   "Failed to read from qdisk within "
+				   "%d second%s (%d.%06d) - REBOOTING\n",
+    				   (int)maxtime.tv_sec,
+    				   maxtime.tv_sec==1?"":"s",
+    				   (int)diff.tv_sec,
+    				   (int)diff.tv_usec);
+			if (!(ctx->qc_flags & RF_DEBUG)) 
+				reboot(RB_AUTOBOOT);
+		}
+	
 		/*
 		 * Reboot if we didn't send a heartbeat in interval*TKO_COUNT
 		 */
+		_diff_tv(&diff, &oldtime, &newtime);
 		if (_cmp_tv(&maxtime, &diff) == 1 &&
 		    ctx->qc_flags & RF_PARANOID) {
 			logt_print(LOG_EMERG, "Failed to complete a cycle within "
@@ -1366,6 +1408,16 @@ get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 		free(val);
 	}
 	
+	/* default = off, so, 1 to turn on */
+	snprintf(query, sizeof(query), "/cluster/quorumd/@io_timeout");
+	if (ccs_get(ccsfd, query, &val) == 0) {
+		if (!atoi(val))
+			ctx->qc_flags &= ~RF_IOTIMEOUT;
+		else
+			ctx->qc_flags |= RF_IOTIMEOUT;
+		free(val);
+	}
+  	
 	/*
 	 * Get flag to see if we're supposed to reboot if we can't complete
 	 * a pass in failure time
@@ -1397,12 +1449,15 @@ get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 	/*
 	 * How many consecutive error cycles do we allow before
 	 * giving up?
+	 *
+	 * Notice that max_error_cycles is disabled if io_timeout is
+	 * active.
 	 */
 	/* default = no max */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@max_error_cycles");
 	if (ccs_get(ccsfd, query, &val) == 0) {
 		ctx->qc_max_error_cycles = atoi(val);
-		if (ctx->qc_max_error_cycles <= 0)
+		if ((ctx->qc_max_error_cycles <= 0) || (ctx->qc_flags & RF_IOTIMEOUT))
 			ctx->qc_max_error_cycles = 0;
 		free(val);
 	}
