@@ -12,7 +12,7 @@
 ##
 #####
 
-import sys, re, pexpect
+import sys, re, pexpect, exceptions
 sys.path.append("@FENCEAGENTSLIBDIR@")
 from fencing import *
 
@@ -27,15 +27,18 @@ def get_power_status(conn, options):
 		node_cmd = "system:blade\[" + options["-n"] + "\]>"
 
 		conn.send("env -T system:blade[" + options["-n"] + "]\r\n")
-		i = conn.log_expect(options, [ node_cmd, "system>" ] , SHELL_TIMEOUT)
+		i = conn.log_expect(options, [ node_cmd, "system>" ] , int(options["-Y"]))
 		if i == 1:
 			## Given blade number does not exist
-			fail(EC_STATUS)
+			if options.has_key("-M"):
+				return "off"
+			else:
+				fail(EC_STATUS)
 		conn.send("power -state\r\n")
-		conn.log_expect(options, node_cmd, SHELL_TIMEOUT)
+		conn.log_expect(options, node_cmd, int(options["-Y"]))
 		status = conn.before.splitlines()[-1]
 		conn.send("env -T system\r\n")
-		conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
+		conn.log_expect(options, options["-c"], int(options["-Y"]))
 	except pexpect.EOF:
 		fail(EC_CONNECTION_LOST)
 	except pexpect.TIMEOUT:
@@ -53,11 +56,11 @@ def set_power_status(conn, options):
 		node_cmd = "system:blade\[" + options["-n"] + "\]>"
 
 		conn.send("env -T system:blade[" + options["-n"] + "]\r\n")
-		conn.log_expect(options, node_cmd, SHELL_TIMEOUT)
+		conn.log_expect(options, node_cmd, int(options["-Y"]))
 		conn.send("power -"+options["-o"]+"\r\n")
-		conn.log_expect(options, node_cmd, SHELL_TIMEOUT)
+		conn.log_expect(options, node_cmd, int(options["-Y"]))
 		conn.send("env -T system\r\n")
-		conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
+		conn.log_expect(options, options["-c"], int(options["-Y"]))
 	except pexpect.EOF:
 		fail(EC_CONNECTION_LOST)
 	except pexpect.TIMEOUT:
@@ -69,9 +72,9 @@ def get_blades_list(conn, options):
 		node_cmd = "system>"
 
 		conn.send("env -T system\r\n")
-		conn.log_expect(options, node_cmd, SHELL_TIMEOUT)
+		conn.log_expect(options, node_cmd, int(options["-Y"]))
 		conn.send("list -l 2\r\n")
-		conn.log_expect(options, node_cmd, SHELL_TIMEOUT)
+		conn.log_expect(options, node_cmd, int(options["-Y"]))
 
 		lines = conn.before.split("\r\n")
 		filter_re = re.compile("^\s*blade\[(\d+)\]\s+(.*?)\s*$")
@@ -91,23 +94,29 @@ def main():
 	device_opt = [  "help", "version", "agent", "quiet", "verbose", "debug",
 			"action", "ipaddr", "login", "passwd", "passwd_script",
 			"cmd_prompt", "secure", "port", "identity_file", "separator",
-			"inet4_only", "inet6_only", "ipport" ]
+			"inet4_only", "inet6_only", "ipport",
+			"power_timeout", "shell_timeout", "login_timeout", "power_wait", "missing_as_off" ]
 
 	atexit.register(atexit_handler)
 
+	all_opt["power_wait"]["default"] = "5"
+	all_opt["cmd_prompt"]["default"] = "system>"
+
 	options = check_input(device_opt, process_input(device_opt))
 
-	## 
-	## Fence agent specific defaults
-	#####
-	if 0 == options.has_key("-c"):
-		options["-c"] = "system>"
-
+	docs = { }        
+	docs["shortdesc"] = "Fence agent for IBM BladeCenter"
+	docs["longdesc"] = "fence_bladecenter is an I/O Fencing agent \
+which can be used with IBM Bladecenters with recent enough firmware that \
+includes telnet support. It logs into a Brocade chasis via telnet or ssh \
+and uses the command line interface to power on and off blades."
+	show_docs(options, docs)
+	
 	##
 	## Operate the fencing device
 	######
 	conn = fence_login(options)
-	fence_action(conn, options, set_power_status, get_power_status, get_blades_list)
+	result = fence_action(conn, options, set_power_status, get_power_status, get_blades_list)
 
 	##
 	## Logout from system
@@ -119,6 +128,8 @@ def main():
 		pass
 	except pexpect.ExceptionPexpect:
 		pass
+	
+	sys.exit(result)
 
 if __name__ == "__main__":
 	main()
