@@ -31,6 +31,19 @@
 
 #define DEFAULT_METHOD "onoff"
 
+/* We should follow FenceAgentsAPI standard*/
+#define ERR_OFF_SUCCESSFUL 0
+#define ERR_OFF_FAIL       1
+
+#define ERR_ON_SUCCESSFUL 0
+#define ERR_ON_FAIL       1
+
+#define ERR_STATUS_ON     0
+#define ERR_STATUS_FAIL   1
+#define ERR_STATUS_OFF    2
+
+#define ERR_OK            0
+
 #define log(lvl, fmt, args...) fprintf(stderr, fmt, ##args)
 #include <libgen.h>
 #include "copyright.cf"
@@ -134,21 +147,26 @@ static struct Etoken power_status[] = {
 /* Structure describing one xml metadata value*/
 struct xml_parameter_s {
   const char *name;
+  const char *getopt;
+  const int required;
+  const char *content_type;
+  const char *default_value;
   const char *description;
 };
 
 /* Array of xml metadatas*/
 struct xml_parameter_s xml_parameters[]={
-  {"auth","IPMI Lan Auth type (md5, password, or none)"},
-  {"ipaddr","IPMI Lan IP to talk to"},
-  {"passwd","Password (if required) to control power on IPMI device"},
-  {"passwd_script","Script to retrieve password (if required)"},
-  {"lanplus","Use Lanplus"},
-  {"login","Username/Login (if required) to control power on IPMI device"},
-  {"action","Operation to perform. Valid operations: on, off, reboot, status, list, monitor or metadata"},
-  {"timeout","Timeout (sec) for IPMI operation"},
-  {"cipher","Ciphersuite to use (same as ipmitool -C parameter)"},
-  {"verbose","Verbose mode"}};
+  {"auth","-A",0,"string",NULL,"IPMI Lan Auth type (md5, password, or none)"},
+  {"ipaddr","-a",1,"string",NULL,"IPMI Lan IP to talk to"},
+  {"passwd","-p",0,"string",NULL,"Password (if required) to control power on IPMI device"},
+  {"passwd_script","-S",0,"string",NULL,"Script to retrieve password (if required)"},
+  {"lanplus","-P",0,"boolean",NULL,"Use Lanplus"},
+  {"login","-l",0,"string",NULL,"Username/Login (if required) to control power on IPMI device"},
+  {"action","-o",0,"string","reboot","Operation to perform. Valid operations: on, off, reboot, status, list, monitor or metadata"},
+  {"timeout","-t",0,"string",NULL,"Timeout (sec) for IPMI operation"},
+  {"cipher","-C",0,"string",NULL,"Ciphersuite to use (same as ipmitool -C parameter)"},
+  {"method","-M",0,"string",DEFAULT_METHOD,"Method to fence (onoff or cycle)"},
+  {"verbose","-v",0,"boolean",NULL,"Verbose mode"}};
 
 /*
    Search for ipmitool
@@ -807,18 +825,41 @@ static void print_xml_metadata(char *pname) {
   int i;
 
   printf("%s\n","<?xml version=\"1.0\" ?>");
-  printf("%s%s%s\n","<resource-agent name=\"",pname,"\" >");
+  printf("%s%s%s\n","<resource-agent name=\"",pname,"\" shortdesc=\"Fence agent for IPMI over LAN\">");
+  printf("<longdesc>\n");
+  printf("fence_ipmilan is an I/O Fencing agent which can be used with "
+         "machines controlled by IPMI. This agent calls support software "
+         "using ipmitool (http://ipmitool.sf.net/).");
+  printf("</longdesc>\n");
   printf("%s\n","<parameters>");
 
   for (i=0;i<(sizeof(xml_parameters)/sizeof(struct xml_parameter_s));i++) {
     printf("\t<parameter name=\"%s\" unique=\"1\">\n",xml_parameters[i].name);
 
-    printf("\t\t<shortdesc lang=\"C\">");
+    printf("\t\t<getopt mixed=\"%s\" />\n",xml_parameters[i].getopt);
+    if (xml_parameters[i].default_value == NULL) {
+      printf("\t\t<content type=\"%s\" />\n",xml_parameters[i].content_type);
+    } else {
+      printf("\t\t<content type=\"%s\" default=\"%s\"/>\n", \
+          xml_parameters[i].content_type, \
+          xml_parameters[i].default_value );
+    }
+
+    printf("\t\t<shortdesc lang=\"en\">");
     printf("%s",xml_parameters[i].description);
     printf("</shortdesc>\n");
     printf("\t</parameter>\n");
   }
   printf("%s\n","</parameters>");
+  printf("%s\n","<actions>");
+  printf("\t<action name=\"%s\" />\n", "on");
+  printf("\t<action name=\"%s\" />\n", "off");
+  printf("\t<action name=\"%s\" />\n", "reboot");
+  printf("\t<action name=\"%s\" />\n", "status");
+  printf("\t<action name=\"%s\" />\n", "list");
+  printf("\t<action name=\"%s\" />\n", "monitor");
+  printf("\t<action name=\"%s\" />\n", "metadata");
+  printf("%s\n","</actions>");
   printf("%s\n","</resource-agent>");
 }
 
@@ -841,6 +882,7 @@ main(int argc, char **argv)
 	int timeout=DEFAULT_TIMEOUT;
 	int cipher=-1;
 	int print_final_status=1;
+	int translated_ret = -1;
 
 	memset(ip, 0, sizeof(ip));
 	memset(authtype, 0, sizeof(authtype));
@@ -947,6 +989,14 @@ main(int argc, char **argv)
 		}
 	}
 
+	if (!strcasecmp(op, "metadata")) {
+	  print_xml_metadata(pname);
+	  translated_ret = ERR_OK;
+	  ret=0;
+	  print_final_status=0;
+	  goto metaout;
+	}
+
 	/*
 	   Validate the operating parameters
 	 */
@@ -1010,19 +1060,22 @@ main(int argc, char **argv)
 		} else {
 			/* Original onoff method */
 			ret = ipmi_off(i);
-			if (ret != 0)
+			translated_ret = (ret==0?ERR_OFF_SUCCESSFUL:ERR_OFF_FAIL);
+			if (ret != 0) {
 				goto out;
+			}
 			ret = ipmi_on(i);
 		}
 	} else if (!strcasecmp(op, "on")) {
 		printf("Powering on machine @ IPMI:%s...", ip);
 		fflush(stdout);
 		ret = ipmi_on(i);
-
+		translated_ret = (ret==0?ERR_ON_SUCCESSFUL:ERR_ON_FAIL);
 	} else if (!strcasecmp(op, "off")) {
 		printf("Powering off machine @ IPMI:%s...", ip);
 		fflush(stdout);
 		ret = ipmi_off(i);
+		translated_ret = (ret==0?ERR_ON_SUCCESSFUL:ERR_ON_FAIL);
 	} else if (!strcasecmp(op, "status") || !strcasecmp(op, "monitor")) {
 		printf("Getting status of IPMI:%s...",ip);
 		fflush(stdout);
@@ -1031,26 +1084,26 @@ main(int argc, char **argv)
 		case STATE_ON:
 		  if (!strcasecmp(op, "status"))
 			    printf("Chassis power = On\n");
+			translated_ret = ERR_STATUS_ON;
 			ret = 0;
 			break;
 		case STATE_OFF:
 		  if (!strcasecmp(op, "status"))
 			    printf("Chassis power = Off\n");
+			translated_ret = ERR_STATUS_OFF;
 			ret = 0;
 			break;
 		default:
 		  if (!strcasecmp(op, "status"))
 			    printf("Chassis power = Unknown\n");
+			translated_ret = ERR_STATUS_FAIL;
 			ret = 1;
 			break;
 		}
 	} else if (!strcasecmp(op, "list")) {
 	  printf("%s\n","N/A");
 	  ret=0;
-	  print_final_status=0;
-	} else if (!strcasecmp(op, "metadata")) {
-	  print_xml_metadata(pname);
-	  ret=0;
+	  translated_ret = ERR_OK;
 	  print_final_status=0;
 	}
 
@@ -1059,11 +1112,12 @@ out:
 	ipmi_destroy(i);
 	free(i);
 
+metaout:
 	if (print_final_status) {
 	  if (ret == 0)
 	    printf("Done\n");
 	  else
 	    printf("Failed\n");
 	}
-	return ret;
+	return translated_ret;
 }
