@@ -1,18 +1,6 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**  
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include "do_rack.h"
 
-char *pname = "fence_rack";
+const char *pname = "fence_rack";
 
 int quiet_flag = 0;
 int verbose_flag = 0;
@@ -148,7 +136,7 @@ void ignore_message_status(void)
 
 
 
-void print_usage(void)
+static void print_usage(void)
 {
   printf("Usage:\n");
   printf("\n");
@@ -168,7 +156,7 @@ void print_usage(void)
 
 
 
-void get_options(int argc, char **argv)
+static void get_options(int argc, char **argv)
 {
   int c;
   char *value;
@@ -249,9 +237,13 @@ void get_options(int argc, char **argv)
       }
       *value = 0;
       if( (value = strchr(arg, '=')) == NULL){
+        /* in this agent we can ignore options without value
+         * as we don't need any of them, here. But we should
+         * accept and ignore them.
+         */
         fprintf(stderr, "invalid input: '%s'\n", arg);
-	exit(DID_FAILURE); 
-      }   
+        continue;
+      }
       *value = 0;
       value++;
     /*  bahfuck. "agent" is not passed to us anyway
