@@ -25,8 +25,6 @@
 #include "options.h"
 #include "debug.h"
 
-LOGSYS_DECLARE_SUBSYS("XVM", SYSLOGLEVEL);
-
 
 /* Assignment functions */
 
@@ -36,6 +34,8 @@ assign_debug(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	if (!value) {
 		/* GNU getopt sets optarg to NULL for options w/o a param
 		   We rely on this here... */
+		/* Command-line debug sets a special flag */
+		args->flags |= F_DEBUG;
 		args->debug++;
 		return;
 	}
@@ -66,7 +66,7 @@ assign_family(fence_xvm_args_t *args, struct arg_info *arg,
 	} else if (!strcasecmp(value, "auto")) {
 		args->family = 0;
 	} else {
-		log_printf(LOG_ERR, "Unsupported family: '%s'\n", value);
+		logt_print(LOG_ERR, "Unsupported family: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -97,7 +97,7 @@ assign_port(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->port = atoi(value);
 	if (args->port <= 0 || args->port >= 65500) {
-		log_printf(LOG_ERR, "Invalid port: '%s'\n", value);
+		logt_print(LOG_ERR, "Invalid port: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -115,7 +115,7 @@ assign_retrans(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->retr_time = atoi(value);
 	if (args->retr_time <= 0) {
-		log_printf(LOG_ERR, "Invalid retransmit time: '%s'\n", value);
+		logt_print(LOG_ERR, "Invalid retransmit time: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -132,7 +132,7 @@ assign_hash(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	} else if (!strcasecmp(value, "sha512")) {
 		args->hash = HASH_SHA512;
 	} else {
-		log_printf(LOG_ERR, "Unsupported hash: %s\n", value);
+		logt_print(LOG_ERR, "Unsupported hash: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -150,7 +150,7 @@ assign_auth(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	} else if (!strcasecmp(value, "sha512")) {
 		args->auth = AUTH_SHA512;
 	} else {
-		log_printf(LOG_ERR, "Unsupported auth type: %s\n", value);
+		logt_print(LOG_ERR, "Unsupported auth type: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -165,7 +165,7 @@ assign_key(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	args->key_file = strdup(value);
 
 	if (stat(value, &st) == -1) {
- 		log_printf(LOG_ERR, "Invalid key file: '%s' (%s)\n", value,
+ 		logt_print(LOG_ERR, "Invalid key file: '%s' (%s)\n", value,
   		       strerror(errno));
 		args->flags |= F_ERR;
 	}
@@ -181,8 +181,10 @@ assign_op(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 		args->op = FENCE_OFF;
 	} else if (!strcasecmp(value, "reboot")) {
 		args->op = FENCE_REBOOT;
+	} else if (!strcasecmp(value, "metadata")) {
+		args->flags |= F_METADATA;
 	} else {
-		log_printf(LOG_ERR, "Unsupported operation: %s\n", value);
+		logt_print(LOG_ERR, "Unsupported operation: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -192,7 +194,7 @@ static inline void
 assign_domain(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	if (args->domain) {
-		log_printf(LOG_ERR,
+		logt_print(LOG_ERR,
 		   "Domain/UUID may not be specified more than once\n");
 		args->flags |= F_ERR;
 		return;
@@ -203,13 +205,13 @@ assign_domain(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	args->domain = strdup(value);
 
 	if (strlen(value) <= 0) {
-		log_printf(LOG_ERR, "Invalid domain name\n");
+		logt_print(LOG_ERR, "Invalid domain name\n");
 		args->flags |= F_ERR;
 	}
 
 	if (strlen(value) >= MAX_DOMAINNAME_LENGTH) {
 		errno = ENAMETOOLONG;
-		log_printf(LOG_ERR, "Invalid domain name: '%s' (%s)\n",
+		logt_print(LOG_ERR, "Invalid domain name: '%s' (%s)\n",
 		       value, strerror(errno));
 		args->flags |= F_ERR;
 	}
@@ -235,7 +237,7 @@ assign_timeout(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->timeout = atoi(value);
 	if (args->timeout <= 0) {
-		log_printf(LOG_ERR, "Invalid timeout: '%s'\n", value);
+		logt_print(LOG_ERR, "Invalid timeout: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -265,6 +267,7 @@ assign_noccs(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 static inline void
 assign_nocluster(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
+	args->flags |= F_NOCCS;
 	args->flags |= F_NOCLUSTER;
 }
 
@@ -283,99 +286,122 @@ assign_uri(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 /** ALL valid command line and stdin arguments for this fencing agent */
 static struct arg_info _arg_info[] = {
 	{ '\xff', NULL, "agent",
+	  0, "boolean", NULL,
 	  "Not user serviceable",
 	  NULL },
 
 	{ '\xff', NULL, "self",
+	  0, "boolean", NULL,
 	  "Not user serviceable", 
 	  NULL },
 
 	{ 'd', "-d", "debug",
+	  0, "boolean", NULL,
 	  "Specify (CCS) / increment (command line) debug level",
 	  assign_debug },
 
 	{ 'f', "-f", NULL,
+	  0, "boolean", NULL,
 	  "Foreground mode (do not fork)",
 	  assign_foreground },
 
 	{ 'i', "-i <family>", "ip_family",
+	  0, "string", "auto",
 	  "IP Family ([auto], ipv4, ipv6)",
 	  assign_family },
 
 	{ 'a', "-a <address>", "multicast_address",
+	  0, "string", NULL,
 	  "Multicast address (default=225.0.0.12 / ff02::3:1)",
 	  assign_address },
 
 	{ 'T', "-T <ttl>", "multicast_ttl",
+	  0, "string", "2",
 	  "Multicast time-to-live (in hops; default=2)",
 	  assign_ttl },
 
 	{ 'p', "-p <port>", "port",
+	  0, "string", "1229",
 	  "IP port (default=1229)",
 	  assign_port },
 
-	{ 'I', "-I <interface>", "multicast_address",
+	{ 'I', "-I <interface>", "interface",
+	  0, "string", NULL,
 	  "Network interface name to listen on",
 	  assign_interface },
 
 	{ 'r', "-r <retrans>", "retrans", 
+	  0, "string", "20",
 	  "Multicast retransmit time (in 1/10sec; default=20)",
 	  assign_retrans },
 
 	{ 'c', "-c <hash>", "hash",
+	  0, "string", "sha256",
 	  "Packet hash strength (none, sha1, [sha256], sha512)",
 	  assign_hash },
 
 	{ 'C', "-C <auth>", "auth",
+	  0, "string", "sha256",
 	  "Authentication (none, sha1, [sha256], sha512)",
 	  assign_auth },
 
 	{ 'k', "-k <file>", "key_file",
+	  0, "string", DEFAULT_KEY_FILE,
 	  "Shared key file (default=" DEFAULT_KEY_FILE ")",
 	  assign_key },
 
 	{ 'o', "-o <operation>", "option",
+	  0, "string", "reboot",
 	  "Fencing operation (null, off, [reboot])",
 	  assign_op },
 
 	{ 'H', "-H <domain>", "domain",
+	  1, "string", NULL,
 	  "Xen host (domain name) to fence",
 	  assign_domain },
 
 	{ 'u', "-u", "use_uuid",
+	  0, "string", NULL,
 	  "Treat <domain> as UUID instead of domain name",
 	  assign_uuid_lookup },
 
 	{ 't', "-t <timeout>", "timeout",
+	  0, "string", "30",
 	  "Fencing timeout (in seconds; default=30)",
 	  assign_timeout },
 
 	{ 'h', "-h", NULL,
+	  0, "boolean", NULL,
  	  "Help",
 	  assign_help },
 
 	{ '?', "-?", NULL,
+	  0, "boolean", NULL,
  	  "Help (alternate)", 
 	  assign_help },
 
 	{ 'X', "-X", NULL,
+	  0, "boolean", NULL,
  	  "Do not connect to CCS for configuration", 
 	  assign_noccs }, 
 
 	{ 'L', "-L", NULL,
- 	  "Local mode only (no cluster)",
+	  0, "boolean", NULL,
+ 	  "Local mode only (no cluster; implies -X)",
 	  assign_nocluster }, 
 
 	{ 'U', "-U", "uri",
+	  0, "string", DEFAULT_HYPERVISOR_URI,
 	  "URI for Hypervisor (default: " DEFAULT_HYPERVISOR_URI ")",
 	  assign_uri },
 	  
 	{ 'V', "-V", NULL,
+	  0, "string", NULL,
  	  "Display version and exit", 
 	  assign_version },
 
 	/* Terminator */
-	{ 0, NULL, NULL, NULL, NULL }
+	{ 0, NULL, NULL, 0, NULL, NULL, NULL, NULL }
 };
 
 
@@ -471,15 +497,13 @@ args_print(fence_xvm_args_t *args)
   Print out arguments and help information based on what is allowed in
   the getopt string optstr.
 
-  TODO use log_printf instead of printf 
-
   @param progname	Program name.
   @param optstr		Getopt(3) style options string
   @param print_stdin	0 = print command line options + description,
 			1 = print fence-style stdin args + description
  */
 void
-args_usage(char *progname, char *optstr, int print_stdin)
+args_usage(char *progname, const char *optstr, int print_stdin)
 {
 	int x;
 	struct arg_info *arg;
@@ -511,6 +535,72 @@ args_usage(char *progname, char *optstr, int print_stdin)
 }
 
 
+static void
+print_desc_xml(const char *desc)
+{
+	const char *d;
+
+	for (d = desc; *d; d++) {
+		switch (*d) {
+		case '<':
+			printf("&lt;");
+			break;
+		case '>':
+			printf("&gt;");
+			break;
+		default:
+			printf("%c", *d);
+		}
+	}
+}
+
+
+void
+args_metadata(char *progname, const char *optstr)
+{
+	int x;
+	struct arg_info *arg;
+
+	printf("<?xml version=\"1.0\" ?>\n");
+	printf("<resource-agent name=\"%s\" shortdesc=\"Fence agent for virtual machines\">\n", basename(progname));
+	printf("<longdesc>fence_xvm  is  an I/O Fencing agent which can be used with any "
+"virtual machines for which libvirt has a backend.  When used on virtual "
+"machines which are backed by clustered storage (and thus, may "
+"be migrated about several physical machines), there is a "
+"requirement that the host machines are also a part of a CMAN/OpenAIS based "
+"cluster, such as linux-cluster.</longdesc>\n");
+	printf("<parameters>\n");
+
+	for (x = 0; x < strlen(optstr); x++) {
+		arg = find_arg_by_char(optstr[x]);
+		if (!arg)
+			continue;
+		if (!arg->stdin_opt)
+			continue;
+
+		printf("\t<parameter name=\"%s\">\n",arg->stdin_opt);
+                printf("\t\t<getopt mixed=\"-%c\" />\n",arg->opt);
+                if (arg->default_value) {
+                  printf("\t\t<content type=\"%s\" default=\"%s\" />\n", arg->content_type, arg->default_value);
+                } else {
+                  printf("\t\t<content type=\"%s\" />\n", arg->content_type);
+                }
+		printf("\t\t<shortdesc lang=\"en\">");
+		print_desc_xml(arg->desc);
+		printf("</shortdesc>\n");
+		printf("\t</parameter>\n");
+	}
+
+	printf("</parameters>\n");
+	printf("<actions>\n");
+	printf("\t<action name=\"off\" />\n");
+	printf("\t<action name=\"reboot\" />\n");
+	printf("\t<action name=\"metadata\" />\n");	
+	printf("</actions>\n");
+	printf("</resource-agent>\n");
+}
+
+
 /**
   Remove leading and trailing whitespace from a line of text.
 
@@ -518,7 +608,7 @@ args_usage(char *progname, char *optstr, int print_stdin)
   @param linelen	Max size of line
   @return		0 on success, -1 on failure
  */
-int
+static int
 cleanup(char *line, size_t linelen)
 {
 	char *p;
@@ -569,7 +659,7 @@ eol:
   @param args		Args structure to fill in.
  */
 void
-args_get_stdin(char *optstr, fence_xvm_args_t *args)
+args_get_stdin(const char *optstr, fence_xvm_args_t *args)
 {
 	char in[256];
 	int line = 0;
@@ -614,7 +704,7 @@ args_get_stdin(char *optstr, fence_xvm_args_t *args)
   @param args		Args structure to fill in.
  */
 void
-args_get_getopt(int argc, char **argv, char *optstr, fence_xvm_args_t *args)
+args_get_getopt(int argc, char **argv, const char *optstr, fence_xvm_args_t *args)
 {
 	int opt;
 	struct arg_info *arg;
@@ -637,7 +727,7 @@ args_get_getopt(int argc, char **argv, char *optstr, fence_xvm_args_t *args)
 void
 args_finalize(fence_xvm_args_t *args)
 {
-	char *addr = NULL;
+	const char *addr = NULL;
 
 	if (!args->addr) {
 		switch(args->family) {
@@ -655,7 +745,7 @@ args_finalize(fence_xvm_args_t *args)
 	}
 
 	if (!args->addr)
-		args->addr = addr;
+		args->addr = strdup(addr);
 
 	if (!args->addr) {
 		printf("No multicast address available\n");
