@@ -2,7 +2,7 @@
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
 #include <linux/spinlock.h>
-#include <asm/semaphore.h>
+#include <linux/semaphore.h>
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
 #include <asm/uaccess.h>
@@ -596,7 +596,7 @@ ea_get_unstuffed(struct gfs_inode *ip, struct gfs_ea_header *ea,
 	unsigned int x;
 	int error = 0;
 
-	bh = kmalloc(nptrs * sizeof(struct buffer_head *), GFP_KERNEL);
+	bh = kmalloc(nptrs * sizeof(struct buffer_head *), GFP_NOFS);
 	if (!bh)
 		return -ENOMEM;
 
@@ -1621,7 +1621,7 @@ ea_acl_chmod_unstuffed(struct gfs_inode *ip,
 	unsigned int x;
 	int error;
 
-	bh = kmalloc(nptrs * sizeof(struct buffer_head *), GFP_KERNEL);
+	bh = kmalloc(nptrs * sizeof(struct buffer_head *), GFP_NOFS);
 	if (!bh)
 		return -ENOMEM;
 
