@@ -300,9 +300,9 @@ static int init_sb(struct gfs_sbd *sdp, int silent, int undo)
 	      sizes, version #s, locations of important on-disk inodes, etc.  */
 
 	error = -EINVAL;
-	if (sdp->sd_sb.sb_bsize < bdev_hardsect_size(sb->s_bdev)) {
+	if (sdp->sd_sb.sb_bsize < bdev_logical_block_size(sb->s_bdev)) {
 		printk("GFS: fsid=%s: FS block size (%u) is too small for device block size (%u)\n",
-		       sdp->sd_fsname, sdp->sd_sb.sb_bsize, bdev_hardsect_size(sb->s_bdev));
+		       sdp->sd_fsname, sdp->sd_sb.sb_bsize, bdev_logical_block_size(sb->s_bdev));
 		goto fail;
 	}
 	if (sdp->sd_sb.sb_bsize > PAGE_SIZE) {
@@ -585,6 +585,17 @@ fail:
 	return error;
 }
 
+void gfs_online_uevent(struct gfs_sbd *sdp)
+{
+	struct super_block *sb = sdp->sd_vfs;
+	char ro[20];
+	char spectator[20];
+	char *envp[] = { ro, spectator, NULL };
+	sprintf(ro, "RDONLY=%d", (sb->s_flags & MS_RDONLY) ? 1 : 0);
+	sprintf(spectator, "SPECTATOR=%d", sdp->sd_args.ar_spectator ? 1 : 0);
+	kobject_uevent_env(&sdp->sd_kobj, KOBJ_ONLINE, envp);
+}
+
 /**
  * fill_super - Read in superblock
  * @sb: The VFS superblock
@@ -638,7 +649,7 @@ static int fill_super(struct super_block *sb, void *data, int silent)
 
 	/*  Copy VFS mount flags  */
 
-	if (sb->s_flags & (MS_NOATIME | MS_NODIRATIME))
+	if (sdp->sd_args.ar_noatime)
 		set_bit(SDF_NOATIME, &sdp->sd_flags);
 	if (sb->s_flags & MS_RDONLY)
 		set_bit(SDF_ROFS, &sdp->sd_flags);
@@ -715,6 +726,7 @@ static int fill_super(struct super_block *sb, void *data, int silent)
 
 	gfs_proc_fs_add(sdp);
 	gfs_glock_dq_uninit(&mount_gh);
+	gfs_online_uevent(sdp);
 
 	return 0;
 
