@@ -16,6 +16,18 @@ int remove_dentry_from_dir(struct gfs2_sbd *sbp, uint64_t dir,
 int find_di(struct gfs2_sbd *sbp, uint64_t childblock, struct dir_info **dip);
 int dinode_hash_insert(osi_list_t *buckets, uint64_t key, struct dir_info *di);
 int dinode_hash_remove(osi_list_t *buckets, uint64_t key);
+int delete_blocks(struct gfs2_inode *ip, uint64_t block,
+		  struct gfs2_buffer_head **bh, const char *btype,
+		  void *private);
+int delete_metadata(struct gfs2_inode *ip, uint64_t block,
+		    struct gfs2_buffer_head **bh, void *private);
+int delete_data(struct gfs2_inode *ip, uint64_t block, void *private);
+int delete_eattr_indir(struct gfs2_inode *ip, uint64_t block, uint64_t parent,
+		       struct gfs2_buffer_head **bh,
+		       enum update_flags *want_updated, void *private);
+int delete_eattr_leaf(struct gfs2_inode *ip, uint64_t block, uint64_t parent,
+		      struct gfs2_buffer_head **bh,
+		      enum update_flags *want_updated, void *private);
 
 /* metawalk_fxns: function pointers to check various parts of the fs
  *
@@ -66,7 +78,8 @@ struct metawalk_fxns {
 				     struct gfs2_ea_header *ea_hdr_prev,
 				     enum update_flags *want_updated,
 				     void *private);
-	int (*finish_eattr_indir) (struct gfs2_inode *ip, int indir_ok,
+	int (*finish_eattr_indir) (struct gfs2_inode *ip, int leaf_pointers,
+				   int leaf_pointer_errors,
 				   enum update_flags *want_updated,
 				   void *private);
 };
